﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using Infragistics.Windows.DataPresenter;
using System.Data;
using System.Collections.ObjectModel;
using System.Windows.Controls.Primitives;
using System.Windows.Threading;
using Infragistics.Controls.Editors;

namespace WpfCustomControlLibrary7
{
    /// <summary>
    /// Follow steps 1a or 1b and then 2 to use this custom control in a XAML file.
    ///
    /// Step 1a) Using this custom control in a XAML file that exists in the current project.
    /// Add this XmlNamespace attribute to the root element of the markup file where it is 
    /// to be used:
    ///
    ///     xmlns:MyNamespace="clr-namespace:WpfCustomControlLibrary7"
    ///
    ///
    /// Step 1b) Using this custom control in a XAML file that exists in a different project.
    /// Add this XmlNamespace attribute to the root element of the markup file where it is 
    /// to be used:
    ///
    ///     xmlns:MyNamespace="clr-namespace:WpfCustomControlLibrary7;assembly=WpfCustomControlLibrary7"
    ///
    /// You will also need to add a project reference from the project where the XAML file lives
    /// to this project and Rebuild to avoid compilation errors:
    ///
    ///     Right click on the target project in the Solution Explorer and
    ///     "Add Reference"->"Projects"->[Select this project]
    ///
    ///
    /// Step 2)
    /// Go ahead and use your control in the XAML file.
    ///
    ///     <MyNamespace:CustomControl1/>
    ///
    /// </summary>
    [TemplatePart(Name = "TestXamData1", Type = typeof(XamDataGrid))]
    public class CustomControl1 : XamDataGrid
    {
        private static RoutedCommand _submitCommand;

        private static RoutedCommand _selectionCommand;

        public ObservableCollection<string> myProperty = new ObservableCollection<string>();

        public ObservableCollection<string> MyProperty
        {
            get
            {
                return myProperty;
            }

            set
            {
                myProperty = value;
            }
        }

        public delegate void CustomEventHandler(object sender, CustomRoutedEventArgs e);

        XamDataGrid Body;

        public List<object> list = new List<object>();

        DataTable dt = new DataTable();

        DataRow dr;

        public FieldLayout fieldLayout = new FieldLayout();

        public static readonly RoutedEvent SubmitEvent = EventManager.RegisterRoutedEvent("Submitted", RoutingStrategy.Bubble, typeof(CustomEventHandler), typeof(CustomControl1));

        public static readonly RoutedEvent SelectedEvent = EventManager.RegisterRoutedEvent("Selected", RoutingStrategy.Bubble, typeof(CustomEventHandler), typeof(CustomControl1));

        static CustomControl1()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(CustomControl1), new FrameworkPropertyMetadata(typeof(CustomControl1)));

            _submitCommand = new RoutedCommand("SubmitCommand", typeof(CustomControl1));

            CommandManager.RegisterClassCommandBinding(typeof(CustomControl1), new CommandBinding(_submitCommand, OnSubmitCommand));

            _selectionCommand = new RoutedCommand("SelectionCommand", typeof(CustomControl1));

            CommandManager.RegisterClassCommandBinding(typeof(CustomControl1), new CommandBinding(_selectionCommand, OnSelectionCommand));           
        }

        public CustomControl1()
        {
      
        }

        public delegate void ClickCommand(Infragistics.Windows.DataPresenter.VirtualizingDataRecordCellPanel SenderCell);

        public void OnButtonClick(object sender)
        {
            //var abc = sender as Infragistics.Windows.DataPresenter.VirtualizingDataRecordCellPanel;            
        }

        public static DependencyProperty SelectionChangedProperty =
                   DependencyProperty.RegisterAttached("SelectionChanged",
                   typeof(ICommand),
                   typeof(CustomControl1),
                   new UIPropertyMetadata(CustomControl1.SelectedItemChanged));

        public static void SetSelectionChanged(DependencyObject target, ICommand value)
        {
            target.SetValue(CustomControl1.SelectionChangedProperty, value);
        }

        private string cmdSelectedValue;

        private string cmdOldValue;

        public string CMBSelectedValue
        {
            get
            {
                return cmdSelectedValue;
            }

            set
            {
                cmdSelectedValue = value;
            }
        }

        public string CMBOldSelectedValue
        {
            get
            {
                return cmdOldValue;
            }

            set
            {
                cmdOldValue = value;
            }
        }

        private static void SelectedItemChanged(DependencyObject target, DependencyPropertyChangedEventArgs e)
        {
            Infragistics.Controls.Editors.XamComboEditor element = target as Infragistics.Controls.Editors.XamComboEditor;

            if (element == null)
            {
                throw new InvalidOperationException("This behavior can be attached to Selector item only.");
            }

            if ((e.NewValue != null) && (e.OldValue == null))
            {
                element.SelectionChanged += new EventHandler(element_SelectionChanged);
            }
            else if ((e.NewValue == null) && (e.OldValue != null))
            {
               // element.SelectionChanged -= SelectionChanged;
            }
        }

        static void element_SelectionChanged(object sender, EventArgs e)
        {
           // throw new NotImplementedException();
            Infragistics.Controls.Editors.XamComboEditor element = sender as Infragistics.Controls.Editors.XamComboEditor;
            ICommand command = (ICommand)element.GetValue(CustomControl1.SelectionChangedProperty);
            command.Execute(element.SelectedItem);
        }

        //private static void element_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        //{
        //    UIElement element = (UIElement)sender;
        //    ICommand command = (ICommand)element.GetValue(CustomControl1.SelectionChangedProperty);
        //    command.Execute(((Selector)sender).SelectedValue);
        //}

        public static RoutedCommand SubmitCommand
        {
            get { return _submitCommand; }
        }

        public static RoutedCommand SelectionCommand
        {
            get { return _selectionCommand; }
        }

        public static void OnSubmitCommand(object sender, ExecutedRoutedEventArgs e)
        {
            CustomControl1 control = sender as CustomControl1;

            if (control != null)
            {
                List<object> list = new List<object>();

                for (int i = 0; i < control.fieldLayout.DataPresenter.ActiveCell.Record.Cells.Count; i++)
                {
                    list.Add(control.fieldLayout.DataPresenter.ActiveCell.Record.Cells[i].Value);
                }

                object fieldName = control.fieldLayout.DataPresenter.ActiveCell.Field;

                control.RaiseSubmitEvent(list,fieldName);

                // string txt = this.dt[control.ActiveCell.Record.DataItemIndex].Status.ToString();

                //Button button = e.OriginalSource as Button;

                //if (button != null)
                //{
                //    MessageBox.Show("Success");
                //}
            }
        }

        public static void OnSelectionCommand(object sender, ExecutedRoutedEventArgs e)
        {
            CustomControl1 control = sender as CustomControl1;

            List<object> list = new List<object>();

            for (int i = 0; i < control.fieldLayout.DataPresenter.ActiveCell.Record.Cells.Count; i++)
            {
                list.Add(control.fieldLayout.DataPresenter.ActiveCell.Record.Cells[i].Value);
            }

           // Data d = new Data();

            string s = ((WpfCustomControlLibrary7.Data)(e.Parameter)).Status.ToString();

            //string s = e.Parameter.ToString();

            if (control != null)
            {
                control.RaiseSelectionEvent(list,s);
            }
        }
        DispatcherTimer dt1;

        public override void OnApplyTemplate()
        {
            LoadStatus();

            base.OnApplyTemplate();

            Body = this.Template.FindName("TestXamData1", this) as XamDataGrid;
         
            Load();
          
            this.Body.FieldLayouts.Add(fieldLayout);
            this.Body.DataContext = dt;

            #region comment

            //Get the list of tags which need for subscription

            //List<object> columnList = new List<object>();

            //for (int i = 0; i < Columns.Count; i++)
            //{
            //    if (Columns[i].IsRequiredforSubscription)
            //    {
            //        columnList.Add(Columns[i].Name);
            //    }
            //}

            #endregion

            //dt.RowChanged += new DataRowChangeEventHandler(dt_RowChanged);

            //dt.ColumnChanged += new DataColumnChangeEventHandler(dt_ColumnChanged);

            dt1 = new DispatcherTimer();
            dt1.Interval = new TimeSpan(0,0,5);
            dt1.Tick += new EventHandler(dt_Tick);
            dt1.Start();

        }

        private void LoadStatus()
        {
            MyProperty.Add("Active");
            MyProperty.Add("InActive");
        }

        void dt_Tick(object sender, EventArgs e)
        {
            Columns[4].IsRequiredforSubscription = true;

            List<object> columnList = new List<object>();

            for (int i = 0; i < Columns.Count; i++)
            {
                if (Columns[i].IsRequiredforSubscription)
                {
                    columnList.Add(Columns[i].Name);
                }
            }

            dt1.Stop();
        }

        //void dt_ColumnChanged(object sender, DataColumnChangeEventArgs e)
        //{
           
        //}

        //void dt_RowChanged(object sender, DataRowChangeEventArgs e)
        //{
        //   // throw new NotImplementedException();
        //}

        private ObservableCollection<ColumnInfo> GetColumnInfo()
        {
            ObservableCollection<ColumnInfo> coulmns = new ObservableCollection<ColumnInfo>();
            ColumnInfo c1, c2, c3, c4, c5;

            TestDataSource t1 = new TestDataSource();


            c1 = new ColumnInfo("Name", DisplayType.TextBlock, new CellInfo(Brushes.Purple),false);
            c2 = new ColumnInfo("Description", DisplayType.TextBlock);
            c3 = new ColumnInfo("SRLoLimit", DisplayType.Button);
            c4 = new ColumnInfo("Status", DisplayType.ComboBox, MyProperty);
            c5 = new ColumnInfo("Value", DisplayType.TextBox);

            coulmns.Add(c1);
            coulmns.Add(c2);
            coulmns.Add(c3);
            coulmns.Add(c4);
            coulmns.Add(c5);

            return coulmns;
        }

        private ObservableCollection<RowInfo> GetRowInfo(ObservableCollection<ColumnInfo> columns)
        {
            ObservableCollection<RowInfo> rows = new ObservableCollection<RowInfo>();

            RowInfo row1 = new RowInfo();

            row1.rowData.Add(new CellData(Brushes.Orange, "CV001", CellData.ErrorType.Warning));
            row1.rowData.Add(new CellData("Sample Description CV001"));
            row1.rowData.Add(new CellData("0"));
            row1.rowData.Add(new CellData("Active"));
            row1.rowData.Add(new CellData(Brushes.Orange, "100", CellData.ErrorType.Warning));

            rows.Add(row1);

            //RowInfo row2 = new RowInfo();

            //row2.rowData.Add("CV002");
            //row2.rowData.Add("Sample Description CV002");
            //row2.rowData.Add("0");
            //row2.rowData.Add("InActive");
            //row2.rowData.Add("200");

            //rows.Add(row2);

            RowInfo row2 = new RowInfo();
            for (int i = 0; i < columns.Count; i++)
            {
                row2.rowData.Add(new CellData(Brushes.OrangeRed, "CV002", CellData.ErrorType.Normal));
                row2.rowData.Add(new CellData(Brushes.Green, "Sample Description CV002", CellData.ErrorType.Normal));
                row2.rowData.Add(new CellData("2"));
                row2.rowData.Add(new CellData("InActive"));
                row2.rowData.Add(new CellData(Brushes.Orange, "200", CellData.ErrorType.Warning));
            }
            rows.Add(row2);

            RowInfo row3 = new RowInfo();

            for (int i = 0; i < columns.Count; i++)
            {
                row3.rowData.Add(new CellData(Brushes.OrangeRed, "CV002", CellData.ErrorType.Normal));
                row3.rowData.Add(new CellData(Brushes.Green, "Sample Description CV002", CellData.ErrorType.Normal));
                row3.rowData.Add(new CellData("2"));
                row3.rowData.Add(new CellData("InActive"));
                row3.rowData.Add(new CellData("200"));
            }
            rows.Add(row3);

            RowInfo row4 = new RowInfo();

            for (int i = 0; i < columns.Count; i++)
            {
                row4.rowData.Add(new CellData(Brushes.OrangeRed, "CV002", CellData.ErrorType.HighValue));
                row4.rowData.Add(new CellData(Brushes.Green, "Sample Description CV002", CellData.ErrorType.Normal));
                row4.rowData.Add(new CellData("2"));
                row4.rowData.Add(new CellData("InActive"));
                row4.rowData.Add(new CellData("200"));
            }
            rows.Add(row4);

            RowInfo row5 = new RowInfo();

            for (int i = 0; i < columns.Count; i++)
            {
                row5.rowData.Add(new CellData(Brushes.OrangeRed, "CV002", CellData.ErrorType.Normal));
                row5.rowData.Add(new CellData(Brushes.Green, "Sample Description CV002", CellData.ErrorType.Normal));
                row5.rowData.Add(new CellData("2"));
                row5.rowData.Add(new CellData("InActive"));
                row5.rowData.Add(new CellData("200"));
            }
            rows.Add(row5);

            RowInfo row6 = new RowInfo();

            for (int i = 0; i < columns.Count; i++)
            {
                row6.rowData.Add(new CellData(Brushes.OrangeRed, "CV002", CellData.ErrorType.UnHealthy));
                row6.rowData.Add(new CellData(Brushes.Green, "Sample Description CV002", CellData.ErrorType.Normal));
                row6.rowData.Add(new CellData("2"));
                row6.rowData.Add(new CellData("InActive"));
                row6.rowData.Add(new CellData("200"));
            }
            rows.Add(row6);

            RowInfo row7 = new RowInfo();

            for (int i = 0; i < columns.Count; i++)
            {
                row7.rowData.Add(new CellData(Brushes.OrangeRed, "CV002", CellData.ErrorType.Warning));
                row7.rowData.Add(new CellData(Brushes.Green, "Sample Description CV002", CellData.ErrorType.Normal));
                row7.rowData.Add(new CellData("2"));
                row7.rowData.Add(new CellData("InActive"));
                row7.rowData.Add(new CellData("200"));
            }
            rows.Add(row3);

            RowInfo row8 = new RowInfo();

            for (int i = 0; i < columns.Count; i++)
            {
                row8.rowData.Add(new CellData(Brushes.OrangeRed, "CV002", CellData.ErrorType.Normal));
                row8.rowData.Add(new CellData(Brushes.Green, "Sample Description CV002", CellData.ErrorType.Normal));
                row8.rowData.Add(new CellData("2"));
                row8.rowData.Add(new CellData("InActive"));
                row8.rowData.Add(new CellData("200"));
            }
            rows.Add(row8);

            //RowInfo row3 = new RowInfo();

            //    row3.rowData.Add("CV003");
            //    row3.rowData.Add("Sample Description CV003");
            //    row3.rowData.Add("0");
            //    row3.rowData.Add("Active");
            //    row3.rowData.Add("100");

            //rows.Add(row3);

            return rows;
        }

        public void Load()
        {
            #region Comment
            //var list = new ObservableCollection<CellInfo>();
            //var rand = new Random();

            //for (Int32 i = 1; i < 4; i++)
            //{
            //    var member = new CellInfo { Name = String.Format("Name {0}", i), Type = DisplayType.TextBlock };
            //    //for (int j = 1; j < 11; j++)
            //    //    member.Periods.Add(new Period { Title = String.Format("Period {0}", j), Hours = rand.Next(0, 160) });
            //    list.Add(member);
            //}

            //CellInfo = list;

            ////a cheat to get the number of columns to create.
            ////var staffMember = this.ViewModel.StaffMembers.First();

            //for (Int32 i = 0; i < CellInfo.Count; i++)
            //{
            //    var field = new UnboundField
            //    {
            //        Name = staffMember.Periods[i].Title,
            //        BindingMode = BindingMode.TwoWay,
            //        BindingPath = new PropertyPath(String.Format("Periods[{0}].Hours", i))
            //    };

            //    field.Settings.EditAsType = typeof(Int32);
            //    // field.Settings.EditorStyle = (Style)Resources["HoursFieldStyle"];

            //    e.FieldLayout.Fields.Add(field);
            //}
            #endregion

            Columns = GetColumnInfo();

            Rows = GetRowInfo(Columns);

            //Updating the columns
            foreach (ColumnInfo item in Columns)
            {
                dt.Columns.Add(item.Name);

                Field field = new Field(item.Name);

                field.Width = new FieldLength(100, FieldLengthUnitType.Auto);

                if (item.Type == DisplayType.TextBox)
                {
                    field.Settings.CellValuePresenterStyle = (Style)Body.FindResource("TextCellStyle");
                    
                }
                else if (item.Type == DisplayType.Button)
                {
                    field.Settings.CellValuePresenterStyle = (Style)Body.FindResource("buttonInCellStyle");
                }
                else if (item.Type == DisplayType.ComboBox)
                {
                     //item.ComboData.Add("ON");
                     //item.ComboData.Add("OFF");

                     //MyProperty = item.ComboData;                     

                     //Style style = new Style(typeof(XamComboEditor));
                     //style.Setters.Add(new Setter(XamComboEditor.ItemsProviderProperty, new DynamicResourceExtension("ComboItemsProvider")));
                     //field.Settings.EditorStyle = style;
                     //field.Settings.EditorType = typeof(XamComboEditor);                                                           

                    field.Settings.CellValuePresenterStyle = (Style)Body.FindResource("comboInCellStyle1");
                }
                else
                {
                    #region comment
                    // field.Settings.CellValuePresenterStyle = (Style)Body.FindResource("InCellStyle");

                    //DataTemplate dataTemplate = new DataTemplate();
                    //Binding valueBinding = new Binding();
                    //valueBinding.Path = new PropertyPath(CellData.ErrorType.Warning);

                    //Binding alignmentBinding = new Binding();
                    //alignmentBinding.Path = new PropertyPath(NewBorderColor);

                    //FrameworkElementFactory textElement = new FrameworkElementFactory(typeof(TextBlock));
                    //textElement.SetBinding(TextBlock.TextProperty, valueBinding);
                    //textElement.SetBinding(TextBlock.TextAlignmentProperty, alignmentBinding);


                    //FrameworkElementFactory gridElement = new FrameworkElementFactory(typeof(Grid));
                    //FrameworkElementFactory borderElement = new FrameworkElementFactory(typeof(Border));

                    // borderElement.AppendChild(textElement);
                    //gridElement.AppendChild(borderElement);
                    //borderElement.SetValue(Border.BorderThicknessProperty, new Thickness(1));

                    //dataTemplate.VisualTree = gridElement;

                    // ApplyRecordAreaStyle();
                    #endregion

                    //cellChangedStyle = new Style(typeof(CellValuePresenter));
                    //cellChangedStyle.Setters.Add(new Setter(CellValuePresenter.BackgroundProperty, Brushes.Red));
                    //cellChangedStyle.Setters.Add(new Setter(CellValuePresenter.ForegroundProperty, Brushes.Yellow));
                    //cellChangedStyle.Setters.Add(new Setter(CellValuePresenter.BorderBrushProperty, Brushes.Green));
                    //cellChangedStyle.Setters.Add(new Setter(CellValuePresenter.BorderHoverBrushProperty, Brushes.Green));
                    //cellChangedStyle.Setters.Add(new Setter(CellValuePresenter.BorderSelectedBrushProperty, Brushes.Green));                   

                    field.Settings.CellValuePresenterStyle = (Style)Body.FindResource("TextBlockCellStyle1");

                    #region Comment
                    //field.Settings.EditorType = typeof(WpfCustomControlLibrary1.CustomTexTEditor);

                    //  field.Settings.EditorType = typeof(TextEditor);
                    //  field.Settings.EditorStyle = (Style)Body.FindResource("style1");

                    // field.Settings.EditorType = typeof(TextEditor);

                    //Style txtEditorStyle = new Style(typeof(TextEditor));

                    //txtEditorStyle.Setters.Add(new Setter(TextEditor.TextProperty, TextValue));
                    #endregion
                    field.Settings.AllowEdit = false;
                }

                fieldLayout.Fields.Add(field);
            }

            //Updating rows

            foreach (RowInfo rowItem in Rows)
            {
                dr = dt.NewRow();

                for (int i = 0; i < Columns.Count; i++)
                {
                    dr[Columns[i].Name] = rowItem.rowData[i].Value;
                }

                dt.Rows.Add(dr);
            }           
        }

        public ObservableCollection<ColumnInfo> Columns
        {
            get { return (ObservableCollection<ColumnInfo>)GetValue(ColumnsProperty); }
            set { SetValue(ColumnsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Columns.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ColumnsProperty =
            DependencyProperty.Register("Columns", typeof(ObservableCollection<ColumnInfo>), typeof(CustomControl1), new UIPropertyMetadata(new ObservableCollection<ColumnInfo>()));


        public ObservableCollection<RowInfo> Rows
        {
            get { return (ObservableCollection<RowInfo>)GetValue(RowsProperty); }
            set { SetValue(RowsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Rows.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty RowsProperty =
            DependencyProperty.Register("Rows", typeof(ObservableCollection<RowInfo>), typeof(CustomControl1), new UIPropertyMetadata(new ObservableCollection<RowInfo>()));

        #region RoutedEvents

        public event CustomEventHandler Submitted
        {
            add { AddHandler(SubmitEvent, value); }
            remove { RemoveHandler(SubmitEvent, value); }
        }

        public void RaiseSubmitEvent(List<object> obj,object fieldName)
        {
            CustomRoutedEventArgs newArgs = new CustomRoutedEventArgs(CustomControl1.SubmitEvent);
            newArgs.RowInfo = obj;
            RaiseEvent(newArgs);
        }

        public event CustomEventHandler Selected
        {
            add { AddHandler(SelectedEvent, value); }
            remove { RemoveHandler(SelectedEvent, value); }
        }

        public void RaiseSelectionEvent(List<object> list, string selectedValue)
        {
            if (selectedValue != CMBSelectedValue)
            {
                CMBOldSelectedValue = CMBSelectedValue;
            }

            CMBSelectedValue = selectedValue;

            CustomRoutedEventArgs newArgs = new CustomRoutedEventArgs(CustomControl1.SelectedEvent);

            newArgs.RowInfo = list;
            newArgs.CurrentValue = CMBSelectedValue;
            newArgs.PrevValue = CMBOldSelectedValue;

            RaiseEvent(newArgs);

        }
        #endregion
    }

    public class TestDataSource
    {
        public ObservableCollection<Data> Status { get; set; }

        public TestDataSource()
        {
            StatusInfo();
        }

        private void StatusInfo()
        {
            var list = new ObservableCollection<Data>();

            list.Add(new Data { Status = "Active" });
            list.Add(new Data { Status = "InActive" });

            Status = list;
        }
    }

    public class Data
    {
        public string Status { get; set; }
    }

    public class ContentToImageConverter : IValueConverter
    {
        //CustomControl1 ctl = new CustomControl1();

        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if ((bool)value)
            {
                return new Uri(@"C:\Users\47700093\Documents\Visual Studio 2010\Projects\WpfCustomControlLibrary7\WpfCustomControlLibrary7\Images\WarningImg.png");

            }
            else
            {
                return null; // new Uri(@"C:\Users\47700093\Documents\Visual Studio 2010\Projects\WpfCustomControlLibrary7\WpfCustomControlLibrary7\Images\WarningImg.png"); // new Uri(@"pack://xx.xx.Servicing.Client;,,/Resources/arrow_left_blue.png");
            }
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class CellValueConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (value == null)
                return Binding.DoNothing;

            if (value is string)
            {
                string temp = (string)value;
                if (temp == "R")
                    return Brushes.Red;
                if (temp == "PG")
                    return Brushes.Gold;
                if (temp == "PG-13")
                    return Brushes.Yellow;
            }

            return Binding.DoNothing;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class BorderConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

}
